+ Signal {
	/** Short Time Fourier Transform
	 * @author Till Bovermann, Thomas Hermann
	 * @date 2006
	 * 
	 * @arg windowSize has to be power of 2
	 * @arg overlap must be smaller then windowSize
	 * @arg windowType  
	 *	[ \hammingWindow, \hanningWindow, \triWindow, \welchWindow, \rectWindow ].choose
	 **/
	stft { |windowSize = 128, overlap = 64, windowType = \hammingWindow|
		var win 		= Signal.perform(windowType, windowSize);
		var cosTab 	= Signal.fftCosTable(windowSize);

		var windowedSig;
		var imag = Signal.newClear(windowSize);
		
		var res;
		
		var hop = windowSize - overlap;
		var idx;	
		
		var fftPart;
		
		(windowSize <= overlap).if{
			// prevent dead-lock
			"Signal-stft: windowSize must be bigger then overlap".warn;
			^nil
		};
		
		res = [];
		
		idx = 0;
		while({(idx + windowSize) < this.size}, {

			windowedSig = win * this[idx .. (idx + windowSize)];
			fftPart = windowedSig.fft(imag, cosTab).magnitude;
			fftPart = fftPart[0 .. ((fftPart.size*0.5).asInteger)];
			res = res ++ [fftPart];

			idx = idx + hop;
		});
		^res;
	}
}